# Generated by Django 5.2 on 2025-04-23 12:46

import django.contrib.postgres.fields
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('claimresponses', '0002_initial'),
        ('claims', '0002_initial'),
        ('encounters', '0002_initial'),
        ('locations', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ExplanationOfBenefitAccident',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('date', models.DateField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitAddItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('item_sequence', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
                ('detail_sequence', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
                ('sub_detail_sequence', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
                ('serviced_date', models.DateField(null=True)),
                ('factor', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('note_number', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitAddItemDetail',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('factor', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('note_number', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitAddItemDetailSubDetail',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('factor', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('note_number', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitBenefitBalance',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('excluded', models.BooleanField(default=False)),
                ('name', models.CharField(max_length=255, null=True)),
                ('description', models.TextField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitBenefitBalanceFinancial',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('allowed_unsigned_int', models.PositiveIntegerField(null=True)),
                ('allowed_string', models.CharField(max_length=255, null=True)),
                ('used_unsigned_int', models.PositiveIntegerField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitCareTeam',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.PositiveIntegerField()),
                ('responsible', models.BooleanField(default=False)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitDiagnosis',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.PositiveIntegerField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitEntererReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitEvent',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('when_date_time', models.DateTimeField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitInsurance',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('focal', models.BooleanField(default=False)),
                ('pre_auth_ref', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.PositiveIntegerField()),
                ('care_team_sequence', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
                ('diagnosis_sequence', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
                ('procedure_sequence', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
                ('information_sequence', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
                ('serviced_date', models.DateField(null=True)),
                ('factor', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('note_number', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitItemAdjudication',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitItemBodySite',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitItemDetail',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.PositiveIntegerField()),
                ('factor', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('note_number', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitItemDetailSubDetail',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.PositiveIntegerField()),
                ('factor', models.DecimalField(decimal_places=2, max_digits=10, null=True)),
                ('note_number', django.contrib.postgres.fields.ArrayField(base_field=models.PositiveIntegerField(), null=True, size=None)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitItemRequestReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitItemReviewOutcome',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('pre_auth_ref', models.CharField(max_length=255)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitPayee',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitPayeePartyReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitPayment',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('date', models.DateField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitPrescriptionReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitProcedure',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.PositiveIntegerField()),
                ('date', models.DateTimeField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitProcessNote',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('number', models.PositiveIntegerField()),
                ('text', models.TextField()),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitProviderReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitRelated',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitSupportingInfo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sequence', models.PositiveIntegerField()),
                ('timing_date', models.DateField(null=True)),
                ('value_boolean', models.BooleanField(null=True)),
                ('value_string', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefitTotal',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ExplanationOfBenefit',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('active', 'Active'), ('cancelled', 'Cancelled'), ('draft', 'Draft'), ('entered-in-error', 'Entered in Error'), ('unknown', 'Unknown')], default='active', max_length=255)),
                ('use', models.CharField(choices=[('claim', 'Claim'), ('preauthorization', 'Preauthorization'), ('predetermination', 'Predetermination')], default='claim', max_length=255)),
                ('created', models.DateTimeField()),
                ('outcome', models.CharField(choices=[('queued', 'Queued'), ('complete', 'Complete'), ('error', 'Error'), ('partial', 'Partial')], default='complete', max_length=255)),
                ('disposition', models.TextField(null=True)),
                ('pre_auth_ref', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=255), null=True, size=None)),
                ('precedence', models.PositiveIntegerField(null=True)),
                ('benefit_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='explanation_of_benefits_benefit_period', to='base.period')),
                ('billable_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='explanation_of_benefits_billable_period', to='base.period')),
                ('claim', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='explanation_of_benefits_claim', to='claims.claimreference')),
                ('claim_response', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='explanation_of_benefits_claim_response', to='claimresponses.claimresponsereference')),
                ('decision', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='explanation_of_benefits_decision', to='base.codeableconcept')),
                ('diagnosis_related_group', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='explanation_of_benefits_diagnosis_related_group', to='base.codeableconcept')),
                ('encounter', models.ManyToManyField(blank=True, related_name='explanation_of_benefits_encounter', to='encounters.encounterreference')),
                ('facility', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='explanation_of_benefits_facility', to='locations.locationorganizationreference')),
                ('form', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='explanation_of_benefits_form', to='base.attachment')),
                ('form_code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='explanation_of_benefits_form_code', to='base.codeableconcept')),
                ('funds_reserve', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='explanation_of_benefits_funds_reserve', to='base.codeableconcept')),
                ('funds_reserve_requested', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='explanation_of_benefits_funds_reserve_requested', to='base.codeableconcept')),
                ('identifier', models.ManyToManyField(blank=True, related_name='explanation_of_benefits_identifier', to='base.identifier')),
                ('insurer', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='explanation_of_benefits_insurer', to='base.organizationreference')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""organization affiliation models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    ExtendedContactDetail,
    Identifier,
    Period,
    TimeStampedModel,
)


class OrganizationAffiliation(TimeStampedModel):
    """organization affiliation model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="organization_affiliations", blank=True
    )
    active = models.BooleanField(default=True)
    period = models.ForeignKey(
        Period,
        on_delete=models.SET_NULL,
        null=True,
        related_name="organization_affiliations_period",
    )
    organization = models.ForeignKey(
        "base.OrganizationReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="organization_affiliations",
    )
    participating_organization = models.ForeignKey(
        "base.OrganizationReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="participating_organization_affiliations",
    )
    network = models.ManyToManyField(
        "base.OrganizationReference",
        blank=True,
        related_name="network_organization_affiliations",
    )
    code = models.ManyToManyField(
        "base.CodeableConcept", blank=True, related_name="organization_affiliation_code"
    )
    specialty = models.ManyToManyField(
        "base.CodeableConcept",
        blank=True,
        related_name="organization_affiliation_specialty",
    )
    location = models.ManyToManyField(
        "locations.LocationReference",
        blank=True,
        related_name="organization_affiliation_location",
    )
    healthcare_service = models.ManyToManyField(
        "healthcareservices.HealthcareServiceReference",
        blank=True,
        related_name="organization_affiliation_healthcare_service",
    )
    contact = models.ManyToManyField(
        ExtendedContactDetail,
        blank=True,
        related_name="organization_affiliation_contact",
    )
    endpoint = models.ManyToManyField(
        "endpoints.EndpointReference",
        blank=True,
        related_name="organization_affiliation_endpoint",
    )


class OrganizationAffiliationReference(BaseReference):
    """organization affiliation reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="organization_affiliation_references_identifier",
    )
    organization_affiliation = models.ForeignKey(
        OrganizationAffiliation,
        on_delete=models.CASCADE,
        related_name="organization_affiliation_references_organization_affiliation",
    )



"""risk assessment urls."""

from django.urls import path

from dfhir.riskassessments.views import (
    RiskAssessmentDetialView,
    RiskManagementListView,
)

app_name = "riskassessments"

urlpatterns = [
    path(
        "riskassessments/",
        RiskManagementListView.as_view(),
        name="risk_management_list",
    ),
    path(
        "riskassessments/<int:pk>/",
        RiskAssessmentDetialView.as_view(),
        name="risk_assessment_detail",
    ),
]



"""Choices for the Flag app."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class FlagStatusChoices(models.TextChoices):
    """Flag status choices."""

    ACTIVE = "active", _("active")
    INACTIVE = "inactive", _("inactive")
    ENTERED_IN_ERROR = "entered-in-error", _("entered-in-error")



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0004_initial'),
        ('careteams', '0003_initial'),
        ('devices', '0001_initial'),
        ('familymemberhistories', '0001_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='familymemberhistory',
            name='patient',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_patient', to='patients.patientreference'),
        ),
        migrations.AddField(
            model_name='familymemberhistory',
            name='relationship',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_relationship', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='familymemberhistory',
            name='sex',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_sex', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='familymemberhistorycondition',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_condition_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='familymemberhistorycondition',
            name='note',
            field=models.ManyToManyField(blank=True, related_name='family_member_history_condition_note', to='base.annotation'),
        ),
        migrations.AddField(
            model_name='familymemberhistorycondition',
            name='onset_age',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_condition_onset_age', to='base.age'),
        ),
        migrations.AddField(
            model_name='familymemberhistorycondition',
            name='onset_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_condition_onset_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='familymemberhistorycondition',
            name='onset_range',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_condition_onset_range', to='base.range'),
        ),
        migrations.AddField(
            model_name='familymemberhistorycondition',
            name='outcome',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_condition_outcome', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='familymemberhistory',
            name='condition',
            field=models.ManyToManyField(blank=True, related_name='family_member_history_condition', to='familymemberhistories.familymemberhistorycondition'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryparticipant',
            name='function',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_participant_function', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='familymemberhistory',
            name='participant',
            field=models.ManyToManyField(blank=True, related_name='family_member_history_participant', to='familymemberhistories.familymemberhistoryparticipant'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryparticipantactorreference',
            name='care_team',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_participant_actor_reference_care_team', to='careteams.careteam'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryparticipantactorreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_participant_actor_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryparticipantactorreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_participant_actor_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryparticipantactorreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_participant_actor_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='familymemberhistoryparticipantactorreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='family_member_history_participant_actor_reference_patient', to='patients.patient'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0002_initial'),
        ('careplans', '0001_initial'),
        ('careteams', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='careplan',
            name='care_team',
            field=models.ManyToManyField(blank=True, related_name='care_plan_care_team', to='careteams.careteamreference'),
        ),
        migrations.AddField(
            model_name='careplan',
            name='category',
            field=models.ManyToManyField(blank=True, related_name='care_plan_category', to='base.codeableconcept'),
        ),
    ]



"""Schedule views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.schedules.models import Schedule
from dfhir.schedules.serializers import ScheduleSerializer

from .filters import ScheduleFilter


class ScheduleListView(APIView):
    """Schedule list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: ScheduleSerializer(many=True)})
    def get(self, request):
        """Get schedules."""
        schedules = Schedule.objects.all()
        schedule_filter = ScheduleFilter(request.GET, queryset=schedules)
        serializer = ScheduleSerializer(schedule_filter.qs, many=True)
        return Response(serializer.data)

    @extend_schema(request=ScheduleSerializer, responses={200: ScheduleSerializer})
    def post(self, request):
        """Create a new schedule."""
        serializer = ScheduleSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class ScheduleDetailView(APIView):
    """Schedule detail view."""

    @extend_schema(responses={200: ScheduleSerializer})
    def get_object(self, pk):
        """Get schedule object."""
        try:
            return Schedule.objects.get(pk=pk)
        except Schedule.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200: ScheduleSerializer})
    def get(self, request, pk=None):
        """Get schedule."""
        schedule = self.get_object(pk)
        serializer = ScheduleSerializer(schedule)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={200: ScheduleSerializer})
    def patch(self, request, pk=None):
        """Update schedule."""
        schedule = self.get_object(pk)
        serializer = ScheduleSerializer(schedule, data=request.data, partial=True)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    def delete(self, request, pk=None):
        """Delete schedule."""
        schedule = self.get_object(pk)
        schedule.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""care plan models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    TimeStampedModel,
)

from . import choices


class CarePlanBasedOnReference(BaseReference):
    """care plan based on reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_based_on_identifier",
    )
    care_plan = models.ForeignKey(
        "CarePlan",
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_based_on_care_plan",
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_based_on_service_request",
    )
    # TODO: request_orientation = models.ForeignKey("RequestOrientation", on_delete=models.SET_NULL, null=True)
    # TODO: nutrition_order = models.ForeignKey("NutritionOrder", on_delete=models.SET_NULL, null=True)


class CarePlanCustodianReference(BaseReference):
    """custodian contributor reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="custodian_contributor_reference_identifier",
    )
    patient = models.ForeignKey(
        "patients.Patient", on_delete=models.SET_NULL, null=True
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner", on_delete=models.SET_NULL, null=True
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole", on_delete=models.SET_NULL, null=True
    )
    device = models.ForeignKey("devices.Device", on_delete=models.SET_NULL, null=True)
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson", on_delete=models.SET_NULL, null=True
    )
    organization = models.ForeignKey(
        "organizations.Organization", on_delete=models.SET_NULL, null=True
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam", on_delete=models.SET_NULL, null=True
    )


class CarePlanContributorReference(CarePlanCustodianReference):
    """contributor reference."""

    pass


class CarePlanAddressReference(BaseReference):
    """care plan address reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_address_reference_identifier",
    )
    condition = models.ForeignKey(
        "conditions.Condition", on_delete=models.SET_NULL, null=True
    )
    medication_administration = models.ForeignKey(
        "medicationadministrations.MedicationAdministration",
        on_delete=models.SET_NULL,
        null=True,
    )


class CarePlanAddressesCodeableReference(TimeStampedModel):
    """care plan addresses codeable reference."""

    reference = models.ForeignKey(
        CarePlanAddressReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_addresses_codeable_reference",
    )
    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_addresses_concept",
    )


class CarePlanPlannedActivityReference(BaseReference):
    """care plan planned activity reference."""

    identifier = models.ForeignKey(Identifier, on_delete=models.SET_NULL, null=True)
    appointment = models.ForeignKey(
        "appointments.Appointment",
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_planned_activity_reference_appointment",
    )
    # TODO: communication_request = models.ForeignKey("communication.CommunicationRequest", on_delete=models.SET_NULL, null=True)
    # TODO: device_request = models.ForeignKey("devicerequests.DeviceRequest", on_delete=models.SET_NULL, null=True)
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_planned_activity_reference_medication_request",
    )
    # TODO: nutrition_order = models.ForeignKey(
    #     "nutritionorders.NutritionOrder",
    #     on_delete=models.SET_NULL,
    #     null=True,
    #     related_name="care_plan_planned_activity_reference_nutrition_order",
    # )
    # TODO: task = models.ForeignKey("tasks.Task", on_delete=models.SET_NULL, null=True)
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_planned_activity_reference_service_request",
    )
    # TODO: vision_prescription = models.ForeignKey("visionprescriptions.VisionPrescription", on_delete=models.SET_NULL, null=True)
    # request_orchestration = models.ForeignKey("requestorchestrations.RequestOrchestration", on_delete=models.SET_NULL, null=True)
    # TODO: immunization_recommendation = models.ForeignKey("immunizationrecommendations.ImmunizationRecommendation", on_delete=models.SET_NULL, null=True)
    # TODO: supply_request = models.ForeignKey("supplyrequests.SupplyRequest", on_delete=models.SET_NULL, null=True)


class CarePlanActivity(TimeStampedModel):
    """care plan activity."""

    performer_activity = models.ManyToManyField(
        "base.CodeableReference",
        blank=True,
        related_name="care_plan_activity_performer_activity",
    )
    progress = models.ManyToManyField(
        "base.Annotation", blank=True, related_name="care_plan_activity_progress"
    )
    planned_activity_reference = models.ForeignKey(
        CarePlanPlannedActivityReference,
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_activity_planned_activity_reference",
    )


class CarePlan(TimeStampedModel):
    """care plan."""

    identifier = models.ManyToManyField(
        Identifier, blank=True, related_name="care_plan_identifier"
    )
    based_on = models.ManyToManyField(
        CarePlanBasedOnReference, blank=True, related_name="care_plan_based_on"
    )
    replaces = models.ManyToManyField(
        "CarePlanReference", blank=True, related_name="care_plan_replaces"
    )
    part_of = models.ManyToManyField(
        "CarePlanReference", blank=True, related_name="care_plan_part_of"
    )
    status = models.CharField(
        max_length=255, null=True, choices=choices.CareplanStatusChoices.choices
    )
    intent = models.CharField(
        max_length=255, null=True, choices=choices.CarePlanIntentChoices.choices
    )
    category = models.ManyToManyField(
        "base.CodeableConcept", blank=True, related_name="care_plan_category"
    )
    title = models.CharField(max_length=255, null=True)
    description = models.TextField(null=True)
    subject = models.ForeignKey(
        "patients.PatientGroupReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_subject",
    )
    encounter = models.ForeignKey(
        "encounters.EncounterReference",
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_encounter",
    )
    period = models.ForeignKey(
        "base.Period",
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_period",
    )
    created = models.DateTimeField(null=True)
    custodian = models.ForeignKey(
        CarePlanCustodianReference,
        null=True,
        related_name="care_plan_custodian",
        on_delete=models.CASCADE,
    )
    contributor = models.ManyToManyField(
        CarePlanContributorReference, blank=True, related_name="care_plan_contributor"
    )
    care_team = models.ManyToManyField(
        "careteams.CareTeamReference", blank=True, related_name="care_plan_care_team"
    )
    addresses = models.ManyToManyField(
        CarePlanAddressesCodeableReference,
        blank=True,
        related_name="care_plan_addresses",
    )
    supporting_info = models.ManyToManyField(
        "base.Reference", blank=True, related_name="care_plan_supporting_info"
    )
    # TODO: goal = models.ManyToManyField(
    #     "base.Reference", blank=True, related_name="care_plan_goal"
    # )
    activity = models.ManyToManyField(
        CarePlanActivity, blank=True, related_name="care_plan_activity"
    )
    note = models.ManyToManyField(
        "base.Annotation", blank=True, related_name="care_plan_note"
    )


class CarePlanReference(BaseReference):
    """care plan reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_reference_identifier",
    )
    care_plan = models.ForeignKey(
        CarePlan,
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_reference_care_plan",
    )


class CarePlanDeviceRequestReference(BaseReference):
    """care plan device request reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_device_request_reference_identifier",
    )
    care_plan = models.ForeignKey(
        CarePlan,
        on_delete=models.SET_NULL,
        null=True,
        related_name="care_plan_device_request_reference_care_plan",
    )
    # device_request = models.ForeignKey(
    #     "devicerequests.DeviceRequest",
    #     on_delete=models.SET_NULL,
    #     null=True,
    #     related_name="care_plan_device_request_reference_device_request",
    # )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('biologicallyderivedproducts', '0002_initial'),
        ('devices', '0001_initial'),
        ('documentreferences', '0005_initial'),
        ('genomicstudies', '0002_initial'),
        ('groups', '0002_initial'),
        ('nutritionproducts', '0001_initial'),
        ('organizations', '0001_initial'),
        ('patients', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('procedures', '0001_initial'),
        ('servicerequests', '0001_initial'),
        ('specimens', '0001_initial'),
        ('substances', '0001_initial'),
        ('tasks', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='genomicstudyanalysis',
            name='specimen',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_analysis_specimen', to='specimens.specimenreference'),
        ),
        migrations.AddField(
            model_name='genomicstudy',
            name='analysis',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_analysis', to='genomicstudies.genomicstudyanalysis'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisdevice',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_device_device', to='devices.devicereference'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisdevice',
            name='function',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_device_function', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysis',
            name='device',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_analysis_device', to='genomicstudies.genomicstudyanalysisdevice'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisinput',
            name='file',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_input_file', to='documentreferences.documentreferencereference'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisinput',
            name='generated_by_identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_input_generated_by_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisinput',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_input_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysis',
            name='input',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_analysis_input', to='genomicstudies.genomicstudyanalysisinput'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisoutput',
            name='file',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_output_file', to='documentreferences.documentreferencereference'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisoutput',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_output_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysis',
            name='output',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_analysis_output', to='genomicstudies.genomicstudyanalysisoutput'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisperformer',
            name='role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_performer_role', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysis',
            name='performer',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_analysis_performer', to='genomicstudies.genomicstudyanalysisperformer'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisperformeractorreference',
            name='device',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_performer_actor_reference_device', to='devices.device'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisperformeractorreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_performer_actor_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisperformeractorreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_performer_actor_reference_organization', to='organizations.organization'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisperformeractorreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_performer_actor_reference_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisperformeractorreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_performer_actor_reference_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisperformer',
            name='actor',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_performer_actor', to='genomicstudies.genomicstudyanalysisperformeractorreference'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisprotocolperformedreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_task_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisprotocolperformedreference',
            name='procedure',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_task_reference_procedure', to='procedures.procedure'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisprotocolperformedreference',
            name='task',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_task_reference_task', to='tasks.task'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysis',
            name='protocol_performed',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_protocol_performed', to='genomicstudies.genomicstudyanalysisprotocolperformedreference'),
        ),
        migrations.AddField(
            model_name='genomicstudybasedonreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_based_on_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='genomicstudybasedonreference',
            name='service_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_based_on_reference_service_request', to='servicerequests.servicerequest'),
        ),
        migrations.AddField(
            model_name='genomicstudybasedonreference',
            name='task',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_based_on_reference_task', to='tasks.task'),
        ),
        migrations.AddField(
            model_name='genomicstudy',
            name='based_on',
            field=models.ManyToManyField(blank=True, related_name='genomic_study_based_on', to='genomicstudies.genomicstudybasedonreference'),
        ),
        migrations.AddField(
            model_name='genomicstudyreference',
            name='genomic_study',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_reference_genomic_study', to='genomicstudies.genomicstudy'),
        ),
        migrations.AddField(
            model_name='genomicstudyreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='genomicstudyanalysisinput',
            name='generated_by_reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_analysis_input_generated_by_reference', to='genomicstudies.genomicstudyreference'),
        ),
        migrations.AddField(
            model_name='genomicstudysubjectreference',
            name='biologically_derived_product',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_subject_reference_biologically_derived_product', to='biologicallyderivedproducts.biologicallyderivedproduct'),
        ),
        migrations.AddField(
            model_name='genomicstudysubjectreference',
            name='group',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_subject_reference_group', to='groups.group'),
        ),
        migrations.AddField(
            model_name='genomicstudysubjectreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_subject_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='genomicstudysubjectreference',
            name='nutrition_product',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_subject_reference_nutrition_product', to='nutritionproducts.nutritionproduct'),
        ),
        migrations.AddField(
            model_name='genomicstudysubjectreference',
            name='patient',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_subject_reference_patient', to='patients.patient'),
        ),
        migrations.AddField(
            model_name='genomicstudysubjectreference',
            name='substance',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_subject_reference_substance', to='substances.substance'),
        ),
        migrations.AddField(
            model_name='genomicstudy',
            name='subject',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='genomic_study_subject', to='genomicstudies.genomicstudysubjectreference'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='DeviceDefinitionHasPart',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('count', models.IntegerField(null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinitionRegulatoryIdentifier',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('type', models.CharField(choices=[('BASIC', 'basic'), ('MASTER', 'master'), ('LICENSE', 'license')], max_length=255)),
                ('device_identifier', models.CharField(max_length=255)),
                ('issuer', models.CharField(max_length=255)),
                ('jurisdiction', models.CharField(max_length=255)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinitionChargeItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('count', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_charge_item_count', to='base.quantity')),
                ('effective_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_charge_item_effective_period', to='base.period')),
                ('use_context', models.ManyToManyField(blank=True, related_name='device_definition_charge_item_use_context', to='base.usagecontext')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinitionClassification',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('justification', models.ManyToManyField(blank=True, related_name='device_definition_classification_jurisdication', to='base.relatedartifact')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_classification_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinitionCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_codeable_reference_concept', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinitionConformsTo',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('version', models.CharField(max_length=255, null=True)),
                ('category', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_conforms_to_category', to='base.codeableconcept')),
                ('source', models.ManyToManyField(blank=True, related_name='device_definition_conforms_to_source', to='base.relatedartifact')),
                ('specification', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_conforms_to_specification', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinitionCorrectiveAction',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('recall', models.BooleanField(null=True)),
                ('scope', models.CharField(choices=[('model', 'model'), ('lot_numbers', 'lot_numbers'), ('serial_numbers', 'serial_numbers')], max_length=255, null=True)),
                ('period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_corrective_action_period', to='base.period')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinitionDeviceName',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=255)),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_device_name_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinitionDeviceVersion',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value', models.CharField(max_length=255, null=True)),
                ('component', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_device_version_component', to='base.identifier')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_device_version_type', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinitionGuideline',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('usage_instruction', models.TextField(null=True)),
                ('intended_use', models.CharField(max_length=255, null=True)),
                ('contraindication', models.ManyToManyField(blank=True, related_name='device_definition_guideline_contraindication', to='base.codeableconcept')),
                ('indication', models.ManyToManyField(blank=True, related_name='device_definition_guideline_indication', to='base.codeableconcept')),
                ('related_artifact', models.ManyToManyField(blank=True, related_name='device_definition_guideline_related_artifact', to='base.relatedartifact')),
                ('use_context', models.ManyToManyField(blank=True, related_name='device_definition_guideline_use_context', to='base.usagecontext')),
                ('warning', models.ManyToManyField(blank=True, related_name='device_definition_guideline_warning', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinitionLink',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('related_device', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_link_related_device', to='devicedefinitions.devicedefinitioncodeablereference')),
                ('relation', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_link_relation', to='base.coding')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinitionMarketDistribution',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('sub_jurisdiction', models.CharField(max_length=255, null=True)),
                ('market_period', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, to='base.period')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinitionMaterial',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('alternate', models.BooleanField(null=True)),
                ('allergenic_indicator', models.BooleanField(null=True)),
                ('substance', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_material_substance', to='base.codeableconcept')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinitionPackagingDistributor',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('name', models.CharField(max_length=255)),
                ('organization_reference', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_packaging_distributor_organization_reference', to='base.organizationreference')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinitionPackaging',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('count', models.IntegerField(null=True)),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_packaging_identifier', to='base.identifier')),
                ('packaging', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_packaging_packaging', to='devicedefinitions.devicedefinitionpackaging')),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_packaging_type', to='base.codeableconcept')),
                ('distributor', models.ManyToManyField(blank=True, related_name='device_definition_packaging_distributor', to='devicedefinitions.devicedefinitionpackagingdistributor')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinitionProperty',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('value_string', models.CharField(max_length=255, null=True)),
                ('value_boolean', models.BooleanField(null=True)),
                ('value_integer', models.IntegerField(null=True)),
                ('type', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_property_type', to='base.codeableconcept')),
                ('value_attachment', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_property_value_attachment', to='base.attachment')),
                ('value_codeable_concept', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_property_value_code', to='base.codeableconcept')),
                ('value_quantity', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_property_value_quantity', to='base.quantity')),
                ('value_range', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_property_value_range', to='base.range')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinition',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('description', models.TextField(null=True)),
                ('part_number', models.CharField(max_length=255, null=True)),
                ('model_number', models.CharField(max_length=255, null=True)),
                ('publisher', models.CharField(max_length=255, null=True)),
                ('contact', models.ManyToManyField(blank=True, related_name='device_definitions_contact', to='base.contactdetail')),
                ('identifier', models.ManyToManyField(blank=True, related_name='device_definitions_identifier', to='base.identifier')),
                ('language_code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definitions_language_code', to='base.codeableconcept')),
                ('manufacturer', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definitions_manufacturer', to='base.organizationreference')),
                ('safety', models.ManyToManyField(blank=True, related_name='device_definitions_safety', to='base.codeableconcept')),
                ('shelf_life_storage', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definitions_shelf_life_storage', to='base.productshelflife')),
                ('charge_item', models.ManyToManyField(blank=True, related_name='device_definitions_charge_item', to='devicedefinitions.devicedefinitionchargeitem')),
                ('classification', models.ManyToManyField(blank=True, related_name='device_definitions_classification', to='devicedefinitions.devicedefinitionclassification')),
                ('conforms_to', models.ManyToManyField(blank=True, related_name='device_definitions_conforms_to', to='devicedefinitions.devicedefinitionconformsto')),
                ('corrective_action', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definitions_corrective_action', to='devicedefinitions.devicedefinitioncorrectiveaction')),
                ('device_name', models.ManyToManyField(blank=True, related_name='device_definitions_device_name', to='devicedefinitions.devicedefinitiondevicename')),
                ('guideline', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definitions_guideline', to='devicedefinitions.devicedefinitionguideline')),
                ('has_part', models.ManyToManyField(blank=True, related_name='device_definitions_has_part', to='devicedefinitions.devicedefinitionhaspart')),
                ('link', models.ManyToManyField(blank=True, related_name='device_definitions_link', to='devicedefinitions.devicedefinitionlink')),
                ('material', models.ManyToManyField(blank=True, related_name='device_definitions_material', to='devicedefinitions.devicedefinitionmaterial')),
                ('packaging', models.ManyToManyField(blank=True, related_name='device_definitions_packaging', to='devicedefinitions.devicedefinitionpackaging')),
                ('property', models.ManyToManyField(blank=True, related_name='device_definitions_property', to='devicedefinitions.devicedefinitionproperty')),
                ('regulatory_identifier', models.ManyToManyField(blank=True, to='devicedefinitions.devicedefinitionregulatoryidentifier')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='DeviceDefinitionReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
                ('device_definition', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_reference_device_definition', to='devicedefinitions.devicedefinition')),
                ('identifier', models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_reference_identifier', to='base.identifier')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='devicedefinitionhaspart',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_has_part_reference', to='devicedefinitions.devicedefinitionreference'),
        ),
        migrations.AddField(
            model_name='devicedefinitioncodeablereference',
            name='reference',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definition_codeable_reference_reference', to='devicedefinitions.devicedefinitionreference'),
        ),
        migrations.CreateModel(
            name='DeviceDefinitionUdiDeviceIdentifier',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('device_identifier', models.CharField(max_length=255)),
                ('issuer', models.CharField(max_length=255)),
                ('jurisdiction', models.CharField(max_length=255)),
                ('market_distribution', models.ManyToManyField(blank=True, to='devicedefinitions.devicedefinitionmarketdistribution')),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.AddField(
            model_name='devicedefinitionpackaging',
            name='udi_device_identifier',
            field=models.ManyToManyField(blank=True, related_name='device_definition_packaging_udihistory', to='devicedefinitions.devicedefinitionudideviceidentifier'),
        ),
        migrations.AddField(
            model_name='devicedefinition',
            name='udi_device_identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='device_definitions_udi_device_identifier', to='devicedefinitions.devicedefinitionudideviceidentifier'),
        ),
    ]



"""Account models."""

from django.db import models

from dfhir.base.models import (
    BaseReference,
    CodeableConcept,
    Identifier,
    Money,
    OrganizationReference,
    Period,
    TimeStampedModel,
)

from . import choices


class AccountSubjectReference(BaseReference):
    """Account subject reference model."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="account_subject_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        related_name="account_subject_reference_patient",
        on_delete=models.CASCADE,
        null=True,
    )
    device = models.ForeignKey(
        "devices.Device",
        related_name="account_subject_reference_device",
        on_delete=models.CASCADE,
        null=True,
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        related_name="account_subject_reference_practitioner",
        on_delete=models.CASCADE,
        null=True,
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        related_name="account_subject_reference_practitioner_role",
        on_delete=models.CASCADE,
        null=True,
    )
    location = models.ForeignKey(
        "locations.Location",
        related_name="account_subject_reference_location",
        on_delete=models.CASCADE,
        null=True,
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthcareService",
        related_name="account_subject_reference_healthcare_service",
        on_delete=models.CASCADE,
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        related_name="account_subject_reference_organization",
        on_delete=models.CASCADE,
        null=True,
    )


class AccountCoverage(TimeStampedModel):
    """Account coverage model."""

    coverage = models.ForeignKey(
        "coverages.CoverageReference",
        related_name="account_coverage_coverage",
        on_delete=models.CASCADE,
        null=True,
    )
    priority = models.PositiveIntegerField(null=True)


class AccountsGuarantorPartyReference(BaseReference):
    """Account guarantor party reference model."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="accounts_guarantor_party_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    patient = models.ForeignKey(
        "patients.Patient",
        related_name="accounts_guarantor_party_reference_patient",
        on_delete=models.CASCADE,
        null=True,
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        related_name="accounts_guarantor_party_reference_related_person",
        on_delete=models.CASCADE,
        null=True,
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        related_name="accounts_guarantor_party_reference_organization",
        on_delete=models.CASCADE,
        null=True,
    )


class AccountGuarantor(TimeStampedModel):
    """Account guarantor model."""

    party = models.ForeignKey(
        AccountsGuarantorPartyReference,
        related_name="account_guarantor_party",
        on_delete=models.CASCADE,
        null=True,
    )
    on_hold = models.BooleanField(default=False)
    period = models.ForeignKey(
        Period,
        related_name="account_guarantor_period",
        on_delete=models.CASCADE,
        null=True,
    )


class AccountDiagnosis(TimeStampedModel):
    """Account diagnosis model."""

    sequence = models.PositiveIntegerField(null=True)
    condition = models.ForeignKey(
        "conditions.ConditionCodeableReference",
        related_name="account_diagnosis_condition",
        on_delete=models.CASCADE,
        null=True,
    )
    date_of_diagnosis = models.DateTimeField(null=True)
    type = models.ManyToManyField(
        CodeableConcept, related_name="account_diagnosis_type", blank=True
    )
    on_admission = models.BooleanField(default=False)
    package_code = models.ManyToManyField(
        CodeableConcept, related_name="account_diagnosis_package_code", blank=True
    )


class AccountProcedure(TimeStampedModel):
    """Account procedure model."""

    sequence = models.PositiveIntegerField(null=True)
    code = models.ForeignKey(
        "procedures.ProcedureCodeableReference",
        related_name="account_procedure_code",
        on_delete=models.CASCADE,
        null=True,
    )
    date_of_service = models.DateTimeField(null=True)
    type = models.ManyToManyField(
        CodeableConcept, related_name="account_procedure_type", blank=True
    )
    package_code = models.ManyToManyField(
        CodeableConcept, related_name="account_procedure_package_code", blank=True
    )
    device = models.ManyToManyField(
        "devices.DeviceReference", related_name="account_procedure_device", blank=True
    )


class AccountRelatedAccount(TimeStampedModel):
    """Account related account model."""

    relationship = models.ForeignKey(
        CodeableConcept,
        related_name="account_related_account_relationship",
        on_delete=models.CASCADE,
        null=True,
    )
    account = models.ForeignKey(
        "AccountReference",
        related_name="account_related_account_reference",
        on_delete=models.CASCADE,
        null=True,
    )


class AccountBalance(TimeStampedModel):
    """Account balance model."""

    aggregate = models.ForeignKey(
        CodeableConcept,
        related_name="account_balance_aggregate",
        on_delete=models.CASCADE,
        null=True,
    )
    term = models.ForeignKey(
        CodeableConcept,
        related_name="account_balance_term",
        on_delete=models.CASCADE,
        null=True,
    )
    estimated = models.BooleanField(default=False)
    amount = models.ForeignKey(
        Money,
        related_name="account_balance_amount",
        on_delete=models.CASCADE,
        null=True,
    )


class Account(TimeStampedModel):
    """Account model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="accounts_identifier", blank=True
    )
    status = models.CharField(
        max_length=255, choices=choices.AccountStatus.choices, null=True
    )
    billing_status = models.ForeignKey(
        CodeableConcept,
        related_name="accounts_billing_status",
        null=True,
        on_delete=models.CASCADE,
    )
    type = models.ForeignKey(
        CodeableConcept,
        related_name="accounts_type",
        null=True,
        on_delete=models.CASCADE,
    )
    name = models.CharField(max_length=255, null=True)
    subject = models.ManyToManyField(
        AccountSubjectReference, related_name="accounts_subject", blank=True
    )
    service_period = models.ForeignKey(
        Period,
        related_name="accounts_service_period",
        on_delete=models.CASCADE,
        null=True,
    )
    covers = models.ManyToManyField(
        "encounters.EncounterEpisodeOfCareReference",
        related_name="accounts_covers",
        blank=True,
    )
    coverage = models.ManyToManyField(
        AccountCoverage, related_name="accounts_coverage", blank=True
    )
    owner = models.ForeignKey(
        OrganizationReference,
        related_name="accounts_owner",
        on_delete=models.CASCADE,
        null=True,
    )
    description = models.TextField(null=True)
    guarantor = models.ManyToManyField(
        AccountGuarantor, related_name="accounts_guarantor", blank=True
    )
    diagnosis = models.ManyToManyField(
        AccountDiagnosis, related_name="accounts_diagnosis", blank=True
    )
    procedure = models.ManyToManyField(
        AccountProcedure, related_name="accounts_procedure", blank=True
    )
    related_account = models.ManyToManyField(
        AccountRelatedAccount, related_name="accounts_related_account", blank=True
    )
    currency = models.ForeignKey(
        CodeableConcept,
        related_name="accounts_currency",
        null=True,
        on_delete=models.CASCADE,
    )
    balance = models.ManyToManyField(
        AccountBalance, related_name="accounts_balance", blank=True
    )
    calculated_at = models.DateTimeField(null=True)


class AccountReference(BaseReference):
    """Account reference model."""

    identifier = models.ForeignKey(
        Identifier,
        related_name="account_reference_identifier",
        on_delete=models.CASCADE,
        null=True,
    )
    account = models.ForeignKey(
        Account,
        related_name="account_reference_account",
        on_delete=models.CASCADE,
        null=True,
    )



"""communications app models."""

from django.db import models

from dfhir.base.models import (
    Annotation,
    Attachment,
    BaseReference,
    CodeableConcept,
    CodeableReference,
    Coding,
    Identifier,
    Reference,
    TimeStampedModel,
)
from dfhir.encounters.models import EncounterReference
from dfhir.patients.models import PatientGroupReference


class CommunicationRequestReference(BaseReference):
    """communication request reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_request_reference_identifier",
    )
    # TODO: communication_request = models.ForeignKey(
    #     "communications.CommunicationRequest",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="communication_request_reference_communication_request",
    # )


class CommunicationRecipientReference(BaseReference):
    """communication recipient reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_recipient_reference_identifier",
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_recipient_reference_care_team",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_recipient_reference_device",
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_recipient_reference_group",
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthcareService",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_recipient_reference_healthcare_service",
    )
    location = models.ForeignKey(
        "locations.Location",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_recipient_reference_location",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_recipient_reference_organization",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_recipient_reference_practitioner_role",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_recipient_reference_related_person",
    )
    endpoint = models.ForeignKey(
        "endpoints.Endpoint",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_recipient_reference_endpoint",
    )


class CommunicationSenderReference(BaseReference):
    """communication sender reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_sender_reference_identifier",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_sender_reference_device",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_sender_reference_organization",
    )
    patient = models.ForeignKey(
        "patients.Patient",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_sender_reference_patient",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_sender_reference_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_sender_reference_practitioner_role",
    )
    related_person = models.ForeignKey(
        "relatedpersons.RelatedPerson",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_sender_reference_related_person",
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthcareService",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_sender_reference_healthcare_service",
    )
    endpoint = models.ForeignKey(
        "endpoints.Endpoint",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_sender_reference_endpoint",
    )
    care_team = models.ForeignKey(
        "careteams.CareTeam",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_sender_reference_care_team",
    )


class CommunicationPayload(TimeStampedModel):
    """communication payload model."""

    content_attachment = models.ForeignKey(
        Attachment,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_payload_content_attachment",
    )
    content_reference = models.ForeignKey(
        Reference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_payload_content_reference",
    )
    content_codeable_concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_payload_content_codeable_concept",
    )


class CommunicationBsedOnReference(BaseReference):
    """communication based on reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_based_on_reference_identifier",
    )
    care_plan = models.ForeignKey(
        "careplans.CarePlan",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_based_on_reference_care_plan",
    )
    # TODO: communication_request = models.ForeignKey(
    #     "communications.CommunicationRequest",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="communication_based_on_reference_communication_request",
    # )
    device_request = models.ForeignKey(
        "devicerequests.DeviceRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_based_on_reference_device_request",
    )
    immunization_recommendation = models.ForeignKey(
        "immunizationrecommendations.ImmunizationRecommendation",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_based_on_reference_immunization_recommendation",
    )
    medication_request = models.ForeignKey(
        "medicationrequests.MedicationRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_based_on_reference_medication_request",
    )
    nutrition_order = models.ForeignKey(
        "nutritionorders.NutritionOrder",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_based_on_reference_nutrition_order",
    )
    service_request = models.ForeignKey(
        "servicerequests.ServiceRequest",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_based_on_reference_service_request",
    )
    task = models.ForeignKey(
        "tasks.Task",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_based_on_reference_task",
    )
    vision_prescription = models.ForeignKey(
        "visionprescriptions.VisionPrescription",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_based_on_reference_vision_prescription",
    )


class Communication(TimeStampedModel):
    """communication model."""

    identifier = models.ManyToManyField(
        Identifier, related_name="communication_identifier", blank=True
    )
    based_on = models.ManyToManyField(
        CommunicationBsedOnReference,
        related_name="communication_based_on",
        blank=True,
    )
    part_of = models.ManyToManyField(
        Reference, related_name="communication_part_of", blank=True
    )
    in_response_to = models.ManyToManyField(
        "communications.CommunicationReference",
        related_name="communication_in_response_to",
        blank=True,
    )
    status = models.ForeignKey(
        Coding,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_status",
    )
    status_reason = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_status_reason",
    )
    category = models.ManyToManyField(
        CodeableConcept, related_name="communication_category", blank=True
    )
    priority = models.ForeignKey(
        Coding,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_priority",
    )
    medium = models.ManyToManyField(
        CodeableConcept, related_name="communication_medium", blank=True
    )
    subject = models.ForeignKey(
        PatientGroupReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_subject",
    )
    topic = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_topic",
    )
    about = models.ManyToManyField(
        Reference, related_name="communication_about", blank=True
    )
    encounter = models.ForeignKey(
        EncounterReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_encounter",
    )
    sent = models.DateTimeField(null=True)
    received = models.DateTimeField(null=True)
    recipient = models.ManyToManyField(
        CommunicationRecipientReference,
        related_name="communication_recipient",
        blank=True,
    )
    sender = models.ForeignKey(
        CommunicationSenderReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_sender",
    )
    reason = models.ManyToManyField(
        CodeableReference, related_name="communication_reason_code", blank=True
    )
    payload = models.ManyToManyField(
        CommunicationPayload, related_name="communication_payload", blank=True
    )
    note = models.ManyToManyField(
        Annotation, related_name="communication_note", blank=True
    )


class CommunicationReference(BaseReference):
    """communication reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_reference_identifier",
    )
    communication = models.ForeignKey(
        Communication,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="communication_reference_communication",
    )



"""Serializers for Conditions."""

from drf_writable_nested.serializers import WritableNestedModelSerializer

from dfhir.base.serializers import (
    AgeSerializer,
    AnnotationSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    CodeableReferenceSerializer,
    IdentifierSerializer,
    PeriodSerializer,
    RangeSerializer,
)
from dfhir.patients.serializers import PatientGroupReferenceSerializer

from .models import (
    Condition,
    ConditionAllergyIntoleranceReference,
    ConditionAsserterReference,
    ConditionCodeableReference,
    ConditionObservationCodeableReference,
    ConditionObservationReference,
    ConditionRecorderReference,
    ConditionReference,
    ConditionStage,
    ConditionStageAssessmentReference,
)


class ConditionObservationReferenceSerializer(BaseReferenceModelSerializer):
    """Condition Observation Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = ConditionObservationReference
        exclude = ["created_at", "updated_at"]


class ConditionObservationCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Condition Observation Codeable Reference Serializer."""

    reference = ConditionObservationReferenceSerializer(many=False, required=False)
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ConditionObservationCodeableReference
        exclude = ["created_at", "updated_at"]


class ConditionStageAssessmentReferenceSerializer(BaseReferenceModelSerializer):
    """Condition Stage Assessment Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = ConditionStageAssessmentReference
        exclude = ["created_at", "updated_at"]


class ConditionStageSerializer(BaseWritableNestedModelSerializer):
    """Condition Stage Serializer."""

    summary = CodeableConceptSerializer(many=False, required=False)
    assessment = ConditionStageAssessmentReferenceSerializer(many=True, required=False)
    type = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ConditionStage
        exclude = ["created_at", "updated_at"]


class ConditionRecorderReferenceSerializer(BaseReferenceModelSerializer):
    """Condition Recorder Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = ConditionRecorderReference
        exclude = ["created_at", "updated_at"]


class ConditionAsserterReferenceSerializer(BaseReferenceModelSerializer):
    """Condition Asserter Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = ConditionAsserterReference
        exclude = ["created_at", "updated_at"]


class ConditionReferenceSerializer(BaseReferenceModelSerializer):
    """Condition Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = ConditionReference
        exclude = ["created_at", "updated_at"]


class ConditionCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Condition Codeable Reference Serializer."""

    reference = ConditionReferenceSerializer(many=False, required=False)
    concept = CodeableConceptSerializer(many=False, required=False)

    class Meta:
        """Meta class."""

        model = ConditionCodeableReference
        exclude = ["created_at", "updated_at"]


class ConditionSerializer(BaseWritableNestedModelSerializer):
    """Condition Serializer."""

    # this method is used to resolve circular import issues with the EncounterReferenceSerializer
    def get_fields(self):
        """Get fields."""
        from dfhir.encounters.serializers import EncounterReferenceSerializer

        fields = super().get_fields()
        fields["encounter"] = EncounterReferenceSerializer(required=False, many=False)
        return fields

    identifier = IdentifierSerializer(many=True, required=False)
    clinical_status = CodeableConceptSerializer(many=False, required=False)
    verification_status = CodeableConceptSerializer(many=False, required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    severity = CodeableConceptSerializer(many=False, required=False)
    code = CodeableConceptSerializer(many=False, required=False)
    body_site = CodeableConceptSerializer(many=True, required=False)
    subject = PatientGroupReferenceSerializer(many=False, required=False)
    onset_age = AgeSerializer(many=False, required=False)
    onset_period = PeriodSerializer(many=False, required=False)
    onset_range = RangeSerializer(many=False, required=False)
    abatement_age = AgeSerializer(many=False, required=False)
    abatement_period = PeriodSerializer(many=False, required=False)
    abatement_range = RangeSerializer(many=False, required=False)
    recorder = ConditionRecorderReferenceSerializer(many=False, required=False)
    asserter = ConditionAsserterReferenceSerializer(many=False, required=False)
    stage = ConditionStageSerializer(many=True, required=False)
    evidence = CodeableReferenceSerializer(many=True, required=False)
    note = AnnotationSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = Condition
        exclude = ["created_at", "updated_at"]


class ConditionAllergyIntoleranceReferenceSerializer(BaseReferenceModelSerializer):
    """Condition Allergy Intolerance Reference Serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = ConditionAllergyIntoleranceReference
        exclude = ["created_at", "updated_at"]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0003_initial'),
        ('chargeitems', '0001_initial'),
        ('encounters', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='chargeitem',
            name='encounter',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='charge_item_encounter', to='encounters.encounterreference'),
        ),
        migrations.AddField(
            model_name='chargeitem',
            name='identifier',
            field=models.ManyToManyField(blank=True, related_name='charge_item_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='chargeitem',
            name='note',
            field=models.ManyToManyField(blank=True, related_name='charge_item_note', to='base.annotation'),
        ),
        migrations.AddField(
            model_name='chargeitem',
            name='occurrence_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='charge_item_occurrence_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='chargeitem',
            name='occurrence_timing',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='charge_item_occurrence_timing', to='base.timing'),
        ),
        migrations.AddField(
            model_name='chargeitem',
            name='override_reason',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='charge_item_override_reason', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='chargeitem',
            name='performing_organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='charge_item_performing_organization', to='base.organizationreference'),
        ),
        migrations.AddField(
            model_name='chargeitem',
            name='quantity',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='charge_item_quantity', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='chargeitem',
            name='requesting_organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='charge_item_requesting_organization', to='base.organizationreference'),
        ),
    ]



"""charge item model choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class ChargeItemStatusChoice(models.TextChoices):
    """charge item status choices."""

    PLANNED = "planned", _("Planned")
    BILLABLE = "billable", _("Billable")
    NOT_BILLABLE = "not-billable", _("Not Billable")
    ABORTED = "aborted", _("Aborted")
    BILLED = "billed", _("Billed")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered in Error")
    UNKNOWN = "unknown", _("Unknown")



"""inventory report choices."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class InventoryReportStatusChoices(models.TextChoices):
    """inventory report status choices."""

    DRAFT = "draft", _("Draft")
    REQUESTED = "requested", _("Requested")
    ACTIVE = "active", _("Active")
    ENTERED_IN_ERROR = "entered-in-error", _("Entered In Error")


class InventoryReportCountTypeChoices(models.TextChoices):
    """inventory report count type choices."""

    SNAPSHOT = "snapshot", _("Snapshot")
    DIFFERENCE = "difference", _("Difference")



"""diagnostic report models."""

from django.db import models

from dfhir.base.models import (
    Annotation,
    Attachment,
    BaseReference,
    CodeableConcept,
    CodeableReference,
    Identifier,
    Period,
    TimeStampedModel,
)
from dfhir.encounters.models import EncounterReference
from dfhir.medicationrequests.models import MedicationRequest
from dfhir.observations.models import Observation, ObservationReference
from dfhir.patients.models import Patient
from dfhir.practitioners.models import Practitioner
from dfhir.servicerequests.models import ServiceRequest

from .choices import DiagnosticReportStatus


class DiagnosticReportCode(TimeStampedModel):
    """diagnostic report code model."""

    display = models.CharField(max_length=255)
    code = models.CharField(max_length=255, null=True)


class DiagnosticCategory(TimeStampedModel):
    """diagnostic report category model."""

    display = models.CharField(max_length=255)
    description = models.TextField(blank=True)


class ConclusionCode(TimeStampedModel):
    """diagnostic report conclusion code model."""

    display = models.CharField(max_length=255)
    code = models.CharField(max_length=255, null=True)


class DiagnosticReportBasedOnReference(BaseReference):
    """diagnostic report based on model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostic_report_based_on_identifier",
    )

    careplan = models.ForeignKey(
        "careplans.CarePlan",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostic_report_based_on_careplan",
    )
    Immunization_recommendation = models.ForeignKey(
        "immunizationrecommendations.ImmunizationRecommendation",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostic_report_based_on_immunization_recommendation",
    )
    medication_request = models.ForeignKey(
        MedicationRequest,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostic_report_based_on_medication_request",
    )
    nutrition_order = models.ForeignKey(
        "nutritionorders.NutritionOrder",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostic_report_based_on_nutrition_order",
    )
    service_request = models.ForeignKey(
        ServiceRequest,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostic_report_based_on_service_request",
    )


class DiagnosticReportSubjectReference(BaseReference):
    """diagnostic report subject model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostic_report_subject_identifier",
    )
    patient = models.ForeignKey(
        Patient,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostic_report_subject_patient",
    )
    location = models.ForeignKey(
        "locations.Location", on_delete=models.DO_NOTHING, null=True
    )
    organization = models.ForeignKey(
        "organizations.Organization", on_delete=models.DO_NOTHING, null=True
    )
    healthcare_service = models.ForeignKey(
        "healthcareservices.HealthcareService",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostic_report_subject_reference_healthcare_service",
    )
    practitioner = models.ForeignKey(
        "practitioners.Practitioner",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostic_report_subject_reference_practitioner",
    )
    medication = models.ForeignKey(
        "medications.Medication",
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="diagnostic_report_subject_reference_medication",
    )
    group = models.ForeignKey(
        "groups.Group",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostic_report_subject_reference_group",
    )
    device = models.ForeignKey(
        "devices.Device",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostic_report_subject_reference_device",
    )
    substance = models.ForeignKey(
        "substances.Substance",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostic_report_subject_reference_substance",
    )
    biologically_derived_product = models.ForeignKey(
        "biologicallyderivedproducts.BiologicallyDerivedProduct",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostic_report_subject_reference_biologically_derived_product",
    )


class DiagnosticReportEffective(TimeStampedModel):
    """diagnostic report effective model."""

    effective_date_time = models.DateTimeField(null=True)
    period = models.ForeignKey(Period, null=True, on_delete=models.DO_NOTHING)


class DiagnosticReportPerformerReference(BaseReference):
    """performer reference."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="performer_identifier",
    )
    practitioner = models.ForeignKey(
        Practitioner,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="performer_practitioner",
    )
    practitioner_role = models.ForeignKey(
        "practitionerroles.PractitionerRole",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="performer_practitioner_role",
    )
    organization = models.ForeignKey(
        "organizations.Organization",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="performer_organization",
    )
    care_plan = models.ForeignKey(
        "careplans.CarePlan",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="performer_care_plan",
    )


class SupportingInfoReference(BaseReference):
    """supporting info reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supporting_info_reference_identifier",
    )
    imaging_study = models.ForeignKey(
        "imagingstudies.ImagingStudy",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supporting_info_imaging_study",
    )
    procedure = models.ForeignKey(
        "procedures.Procedure",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supporting_info_procedure",
    )
    observation = models.ForeignKey(
        Observation,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supporting_info_observation",
    )
    diagnostic_report = models.ForeignKey(
        "DiagnosticReport",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supporting_info_diagnostic_report",
    )
    # TODO: citation = models.ForeignKey("Citation", on_delete=models.DO_NOTHING, null=True)
    family_member_history = models.ForeignKey(
        "familymemberhistories.FamilyMemberHistory",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supporting_info_family_member_history",
    )
    allergy_intolerance = models.ForeignKey(
        "allergyintolerances.AllergyIntolerance",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supporting_info_allergy_intolerance",
    )
    device_usage = models.ForeignKey(
        "deviceusages.DeviceUsage",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supporting_info_device_usage",
    )


class SupportingInfo(TimeStampedModel):
    """diagnostic report supporting info model."""

    type = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="supporting_info_type",
    )
    reference = models.ForeignKey(
        SupportingInfoReference,
        null=True,
        on_delete=models.SET_NULL,
        related_name="supporting_info_reference",
    )


class ConclusionCodeReference(BaseReference):
    """conclusion code reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="conclusion_code_reference_identifier",
    )
    observation = models.ForeignKey(
        "observations.Observation",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="conclusion_code_observation",
    )
    condition = models.ForeignKey(
        "conditions.Condition",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="conclusion_code_condition",
    )


class ConclusionCodeCodeableReference(TimeStampedModel):
    """conclusion code codeable reference model."""

    concept = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="conclusion_code_codeable_reference_concept",
    )
    reference = models.ForeignKey(
        ConclusionCodeReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="conclusion_code_codeable_reference_codeable",
    )


class DiagnosticReportMedia(TimeStampedModel):
    """media model."""

    comment = models.TextField(blank=True)
    link = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostic_report_media_document_reference",
    )


class DiagnosticReport(TimeStampedModel):
    """diagnostic report model."""

    identifier = models.ManyToManyField(
        Identifier,
        blank=True,
        related_name="diagnostic_report_identifier",
    )
    based_on = models.ManyToManyField(
        DiagnosticReportBasedOnReference,
        blank=True,
        related_name="diagnostic_report_based_on",
    )
    status = models.CharField(
        max_length=255, choices=DiagnosticReportStatus.choices, null=True
    )
    category = models.ManyToManyField(
        CodeableConcept, blank=True, related_name="diagnostics_report_category"
    )
    code = models.ForeignKey(
        CodeableConcept,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostics_report_code",
    )
    subject = models.ForeignKey(
        DiagnosticReportSubjectReference, on_delete=models.DO_NOTHING, null=True
    )
    encounter = models.ForeignKey(
        EncounterReference,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="diagnostic_encounter",
    )
    effective_date_time = models.DateTimeField(null=True)
    effective_period = models.ForeignKey(
        Period,
        null=True,
        on_delete=models.DO_NOTHING,
        related_name="diagnostic_report_effective_period",
    )
    issued = models.DateTimeField(null=True)
    performer = models.ManyToManyField(
        DiagnosticReportPerformerReference,
        blank=True,
        related_name="diagnostic_report_performer",
    )
    results_interpretation = models.ManyToManyField(
        DiagnosticReportPerformerReference,
        blank=True,
        related_name="diagnostic_results_interpretation",
    )

    specimen = models.ManyToManyField(
        "specimens.SpecimenReference",
        blank=True,
        related_name="diagnostic_report_specimen",
    )
    result = models.ManyToManyField(
        ObservationReference, blank=True, related_name="diagnostic_result"
    )
    note = models.ManyToManyField(
        Annotation, blank=True, related_name="diagnostic_report_note"
    )
    # study = models.ManyToManyField(
    #     "genomistudy.GenomicImagingStudyReference",
    #     blank=True,
    #     related_name="diagnostic_report_study",
    # )
    supporting_info = models.ManyToManyField(
        SupportingInfo, blank=True, related_name="diagnostic_report_supporting_info"
    )
    media = models.ManyToManyField(DiagnosticReportMedia, blank=True)
    # TODO: composition = models.ForeignKey(
    #     "Composition",
    #     on_delete=models.DO_NOTHING,
    #     null=True,
    #     related_name="diagnostic_report_composition",
    # )
    conclusion = models.TextField(null=True)
    conclusion_code = models.ManyToManyField(
        ConclusionCodeCodeableReference,
        blank=True,
        related_name="diagnostic_report_conclusion_code",
    )
    recommendation = models.ManyToManyField(CodeableReference, blank=True)
    presented_form = models.ManyToManyField(Attachment, blank=True)
    communication = models.ManyToManyField(
        "communications.CommunicationReference",
        blank=True,
        related_name="diagnostic_report_communication",
    )


class DiagnosticReportDocumentReferenceReference(BaseReference):
    """document reference reference model."""

    identifier = models.ForeignKey(
        Identifier,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="document_reference_reference_identifier",
    )
    # TODO: fix!
    document_reference = models.ForeignKey(
        "documentreferences.DocumentReference",
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="document_reference_reference_document_reference",
    )
    diagnostic_report = models.ForeignKey(
        DiagnosticReport,
        on_delete=models.DO_NOTHING,
        null=True,
        related_name="document_reference_reference_diagnostic_report",
    )



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('claimresponses', '0004_initial'),
        ('coverages', '0002_initial'),
        ('practitionerroles', '0002_initial'),
        ('practitioners', '0001_initial'),
        ('servicerequests', '0001_initial'),
        ('supplyrequests', '0001_initial'),
        ('visionprescriptions', '0001_initial'),
    ]

    operations = [
        migrations.AddField(
            model_name='claimresponseadditemrequestreference',
            name='service_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_add_item_request_reference_service_request', to='servicerequests.servicerequest'),
        ),
        migrations.AddField(
            model_name='claimresponseadditemrequestreference',
            name='supply_request',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_add_item_request_reference_supply_request', to='supplyrequests.supplyrequest'),
        ),
        migrations.AddField(
            model_name='claimresponseadditemrequestreference',
            name='vision_prescription',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_add_item_request_reference_vision_prescription', to='visionprescriptions.visionprescription'),
        ),
        migrations.AddField(
            model_name='claimresponseadditem',
            name='request',
            field=models.ManyToManyField(blank=True, related_name='claim_response_add_item_request', to='claimresponses.claimresponseadditemrequestreference'),
        ),
        migrations.AddField(
            model_name='claimresponseerror',
            name='code',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_error_code', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='claimresponse',
            name='error',
            field=models.ManyToManyField(blank=True, related_name='claim_response_error', to='claimresponses.claimresponseerror'),
        ),
        migrations.AddField(
            model_name='claimresponseevent',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_event_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='claimresponseevent',
            name='when_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_event_when_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='claimresponse',
            name='event',
            field=models.ManyToManyField(blank=True, related_name='claim_response_event', to='claimresponses.claimresponseevent'),
        ),
        migrations.AddField(
            model_name='claimresponseinsurance',
            name='coverage',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_insurance_coverage', to='coverages.coveragereference'),
        ),
        migrations.AddField(
            model_name='claimresponse',
            name='insurance',
            field=models.ManyToManyField(blank=True, related_name='claim_response_insurance', to='claimresponses.claimresponseinsurance'),
        ),
        migrations.AddField(
            model_name='claimresponseitem',
            name='trace_number',
            field=models.ManyToManyField(blank=True, related_name='claim_response_item_trace_number', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='claimresponse',
            name='item',
            field=models.ManyToManyField(blank=True, related_name='claim_response_item', to='claimresponses.claimresponseitem'),
        ),
        migrations.AddField(
            model_name='claimresponseitemadjudication',
            name='amount',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_item_adjudication_amount', to='base.money'),
        ),
        migrations.AddField(
            model_name='claimresponseitemadjudication',
            name='category',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_item_adjudication_category', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='claimresponseitemadjudication',
            name='quantity',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_item_adjudication_quantity', to='base.quantity'),
        ),
        migrations.AddField(
            model_name='claimresponseitemadjudication',
            name='reason',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_item_adjudication_reason', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='claimresponseitem',
            name='adjudication',
            field=models.ManyToManyField(blank=True, related_name='claim_response_item_adjudication', to='claimresponses.claimresponseitemadjudication'),
        ),
        migrations.AddField(
            model_name='claimresponseadditemdetailsubdetail',
            name='adjudication',
            field=models.ManyToManyField(blank=True, related_name='claim_response_add_item_detail_sub_detail_adjudication', to='claimresponses.claimresponseitemadjudication'),
        ),
        migrations.AddField(
            model_name='claimresponseadditemdetail',
            name='adjudication',
            field=models.ManyToManyField(blank=True, related_name='claim_response_add_item_detail_adjudication', to='claimresponses.claimresponseitemadjudication'),
        ),
        migrations.AddField(
            model_name='claimresponseadditem',
            name='adjudication',
            field=models.ManyToManyField(blank=True, related_name='claim_response_add_item_adjudication', to='claimresponses.claimresponseitemadjudication'),
        ),
        migrations.AddField(
            model_name='claimresponse',
            name='adjudication',
            field=models.ManyToManyField(blank=True, related_name='claim_response_adjudication', to='claimresponses.claimresponseitemadjudication'),
        ),
        migrations.AddField(
            model_name='claimresponseitemdetail',
            name='adjudication',
            field=models.ManyToManyField(blank=True, related_name='claim_response_item_detail_adjudication', to='claimresponses.claimresponseitemadjudication'),
        ),
        migrations.AddField(
            model_name='claimresponseitemdetail',
            name='trace_number',
            field=models.ManyToManyField(blank=True, related_name='claim_response_item_detail_trace_number', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='claimresponseitem',
            name='detail',
            field=models.ManyToManyField(blank=True, related_name='claim_response_item_detail', to='claimresponses.claimresponseitemdetail'),
        ),
        migrations.AddField(
            model_name='claimresponseitemdetailsubdetail',
            name='adjudication',
            field=models.ManyToManyField(blank=True, related_name='claim_response_item_detail_sub_detail_adjudication', to='claimresponses.claimresponseitemadjudication'),
        ),
        migrations.AddField(
            model_name='claimresponseitemdetailsubdetail',
            name='trace_number',
            field=models.ManyToManyField(blank=True, related_name='claim_response_item_detail_sub_detail_trace_number', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='claimresponseitemdetail',
            name='sub_detail',
            field=models.ManyToManyField(blank=True, related_name='claim_response_item_detail_sub_detail', to='claimresponses.claimresponseitemdetailsubdetail'),
        ),
        migrations.AddField(
            model_name='claimresponseitemreviewoutcome',
            name='decision',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_item_review_outcome_decision', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='claimresponseitemreviewoutcome',
            name='pre_auth_period',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_item_review_outcome_pre_auth_period', to='base.period'),
        ),
        migrations.AddField(
            model_name='claimresponseitemreviewoutcome',
            name='reason',
            field=models.ManyToManyField(blank=True, related_name='claim_response_item_review_outcome_reason', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='claimresponseitemdetailsubdetail',
            name='review_outcome',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_item_detail_sub_detail_review_outcome', to='claimresponses.claimresponseitemreviewoutcome'),
        ),
        migrations.AddField(
            model_name='claimresponseitemdetail',
            name='review_outcome',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_item_detail_review_outcome', to='claimresponses.claimresponseitemreviewoutcome'),
        ),
        migrations.AddField(
            model_name='claimresponseitem',
            name='review_outcome',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_item_review_outcome', to='claimresponses.claimresponseitemreviewoutcome'),
        ),
        migrations.AddField(
            model_name='claimresponseadditemdetailsubdetail',
            name='review_outcome',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_add_item_detail_sub_detail_review_outcome', to='claimresponses.claimresponseitemreviewoutcome'),
        ),
        migrations.AddField(
            model_name='claimresponseadditemdetail',
            name='review_outcome',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_add_item_detail_review_outcome', to='claimresponses.claimresponseitemreviewoutcome'),
        ),
        migrations.AddField(
            model_name='claimresponseadditem',
            name='review_outcome',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_add_item_review_outcome', to='claimresponses.claimresponseitemreviewoutcome'),
        ),
        migrations.AddField(
            model_name='claimresponsepayment',
            name='adjustment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_payment_adjustment', to='base.money'),
        ),
        migrations.AddField(
            model_name='claimresponsepayment',
            name='adjustment_reason',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_payment_adjustment_reason', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='claimresponsepayment',
            name='amount',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_payment_amount', to='base.money'),
        ),
        migrations.AddField(
            model_name='claimresponsepayment',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_payment_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='claimresponsepayment',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_payment_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='claimresponse',
            name='payment',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_payment', to='claimresponses.claimresponsepayment'),
        ),
        migrations.AddField(
            model_name='claimresponseprocessnote',
            name='language',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_process_note_language', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='claimresponseprocessnote',
            name='type',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_process_note_type', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='claimresponse',
            name='process_note',
            field=models.ManyToManyField(blank=True, related_name='claim_response_process_note', to='claimresponses.claimresponseprocessnote'),
        ),
        migrations.AddField(
            model_name='claimresponsereference',
            name='claim_response',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_reference_claim_response', to='claimresponses.claimresponse'),
        ),
        migrations.AddField(
            model_name='claimresponsereference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_reference_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='claimresponseinsurance',
            name='claim_response',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_insurance_claim_response', to='claimresponses.claimresponsereference'),
        ),
        migrations.AddField(
            model_name='claimresponserequestorreference',
            name='identifier',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_requestor_identifier', to='base.identifier'),
        ),
        migrations.AddField(
            model_name='claimresponserequestorreference',
            name='organization',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_requestor_organization', to='base.organizationreference'),
        ),
        migrations.AddField(
            model_name='claimresponserequestorreference',
            name='practitioner',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_requestor_practitioner', to='practitioners.practitioner'),
        ),
        migrations.AddField(
            model_name='claimresponserequestorreference',
            name='practitioner_role',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_requestor_practitioner_role', to='practitionerroles.practitionerrole'),
        ),
        migrations.AddField(
            model_name='claimresponse',
            name='requestor',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_requestor', to='claimresponses.claimresponserequestorreference'),
        ),
        migrations.AddField(
            model_name='claimresponsetotal',
            name='amount',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_total_amount', to='base.money'),
        ),
        migrations.AddField(
            model_name='claimresponsetotal',
            name='category',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.CASCADE, related_name='claim_response_total_category', to='base.codeableconcept'),
        ),
        migrations.AddField(
            model_name='claimresponse',
            name='total',
            field=models.ManyToManyField(blank=True, related_name='claim_response_total', to='claimresponses.claimresponsetotal'),
        ),
    ]



# Generated by Django 5.2 on 2025-04-23 12:46

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('base', '0005_initial'),
        ('bodystructures', '0002_initial'),
        ('conditions', '0004_initial'),
        ('encounters', '0003_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ProcedureBasedOnReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedureCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedureFocalDevice',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedureFocusReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedurePartOfReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedurePerformer',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedurePerformerActorReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedureReasonCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedureReasonReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedureRecorderReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedureReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedureReportedReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedureReportReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedureSubjectReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedureUsedCodeableReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='ProcedureUsedReference',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('reference', models.CharField(max_length=255, null=True)),
                ('type', models.CharField(max_length=255, null=True)),
                ('display', models.CharField(max_length=255, null=True)),
            ],
            options={
                'abstract': False,
            },
        ),
        migrations.CreateModel(
            name='Procedure',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now_add=True)),
                ('updated_at', models.DateTimeField(auto_now=True)),
                ('status', models.CharField(choices=[('preparation', 'Preparation'), ('in-progress', 'In Progress'), ('not-done', 'Not Done'), ('on-hold', 'On Hold'), ('stopped', 'Stopped'), ('completed', 'Completed'), ('entered-in-error', 'Entered in Error'), ('unknown', 'Unknown')], max_length=255, null=True)),
                ('occurrence_date_time', models.DateTimeField(null=True)),
                ('occurrence_string', models.CharField(max_length=255, null=True)),
                ('recorded', models.DateTimeField(null=True)),
                ('reported_boolean', models.BooleanField(null=True)),
                ('body_site', models.ManyToManyField(blank=True, related_name='procedure_body_site', to='base.codeableconcept')),
                ('body_structure', models.ManyToManyField(blank=True, related_name='procedure_body_structure', to='bodystructures.bodystructurereference')),
                ('category', models.ManyToManyField(blank=True, related_name='procedure_category', to='base.codeableconcept')),
                ('code', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_code', to='base.codeableconcept')),
                ('complication', models.ManyToManyField(blank=True, related_name='procedure_complication', to='conditions.conditioncodeablereference')),
                ('encounter', models.ForeignKey(null=True, on_delete=django.db.models.deletion.DO_NOTHING, related_name='procedure_encounter', to='encounters.encounterreference')),
            ],
            options={
                'abstract': False,
            },
        ),
    ]



"""Nutrition Products serializers."""

from drf_writable_nested.serializers import WritableNestedModelSerializer
from rest_framework import serializers

from dfhir.base.serializers import (
    AnnotationSerializer,
    AttachmentSerializer,
    BaseReferenceModelSerializer,
    BaseWritableNestedModelSerializer,
    CodeableConceptSerializer,
    IdentifierSerializer,
    OrganizationReferenceSerializer,
    QuantitySerializer,
    RatioSerializer,
)
from dfhir.substances.serializers import SubstanceCodeableReferenceSerializer

from .models import (
    NutritionProduct,
    NutritionProductCharacteristic,
    NutritionProductCodeableReference,
    NutritionProductIngredient,
    NutritionProductInstance,
    NutritionProductNutrients,
    NutritionProductReference,
)


class NutritionProductReferenceSerializer(BaseReferenceModelSerializer):
    """Nutrition Product Reference serializer."""

    identifier = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = NutritionProductReference
        exclude = ["created_at", "updated_at"]


class NutritionProductCodeableReferenceSerializer(WritableNestedModelSerializer):
    """Nutrition Product Codeable Reference serializer."""

    reference = NutritionProductReferenceSerializer(required=False)
    concept = CodeableConceptSerializer(required=False)

    class Meta:
        """Meta class."""

        model = NutritionProductCodeableReference
        exclude = ["created_at", "updated_at"]


class NutritionProductCharacteristicSerializer(WritableNestedModelSerializer):
    """Nutrition Product Characteristic serializer."""

    type = CodeableConceptSerializer(required=False)
    value_codeable_concept = CodeableConceptSerializer(required=False)
    value_quantity = QuantitySerializer(required=False)
    value_attachment = AttachmentSerializer(required=False)

    class Meta:
        """Meta class."""

        model = NutritionProductCharacteristic
        exclude = ["created_at", "updated_at"]


class NutritionProductInstanceSerializer(WritableNestedModelSerializer):
    """Nutrition Product Instance serializer."""

    quantity = QuantitySerializer(required=False)
    identifier = IdentifierSerializer(many=True, required=False)
    biological_source_event = IdentifierSerializer(required=False)

    class Meta:
        """Meta class."""

        model = NutritionProductInstance
        exclude = ["created_at", "updated_at"]


class NutritionProductNutrientsSerializer(WritableNestedModelSerializer):
    """Nutrition Product Nutrients serializer."""

    item = SubstanceCodeableReferenceSerializer(required=False)
    amount_ratio = RatioSerializer(required=False)
    amount_quantity = QuantitySerializer(required=False)

    class Meta:
        """Meta class."""

        model = NutritionProductNutrients
        exclude = ["created_at", "updated_at"]


class NutritionProductIngredientSerializer(WritableNestedModelSerializer):
    """Nutrition Product Ingredient serializer."""

    item = NutritionProductCodeableReferenceSerializer(required=False)
    amount_ratio = RatioSerializer(required=False)
    amount_quantity = QuantitySerializer(required=False)

    class Meta:
        """Meta class."""

        model = NutritionProductIngredient
        exclude = ["created_at", "updated_at"]


class NutritionProductSerializer(BaseWritableNestedModelSerializer):
    """Nutrition Product serializer."""

    code = CodeableConceptSerializer(required=False)
    status = serializers.CharField(required=False)
    category = CodeableConceptSerializer(many=True, required=False)
    manufacturer = OrganizationReferenceSerializer(many=True, required=False)
    nutrient = NutritionProductNutrientsSerializer(many=True, required=False)
    ingredient = NutritionProductIngredientSerializer(many=True, required=False)
    energy = QuantitySerializer(required=False)
    characteristic = NutritionProductCharacteristicSerializer(many=True, required=False)
    instance = NutritionProductInstanceSerializer(many=True, required=False)
    note = AnnotationSerializer(many=True, required=False)

    class Meta:
        """Meta class."""

        model = NutritionProduct
        exclude = ["created_at", "updated_at"]



"""Device definitions urls."""

from django.urls import path

from . import views

app_name = "devicedefinitions"

urlpatterns = [
    path(
        "devicedefinitions/",
        views.DeviceDefinitionListCreateView.as_view(),
        name="list-create",
    ),
    path(
        "devicedefinitions/<int:pk>/",
        views.DeviceDefinitionDetailView.as_view(),
        name="detail",
    ),
]



"""medication statements views."""

from django.http import Http404
from drf_spectacular.utils import extend_schema
from rest_framework import status
from rest_framework.permissions import AllowAny
from rest_framework.response import Response
from rest_framework.views import APIView

from dfhir.medicationstatements.models import MedicationStatement
from dfhir.medicationstatements.serializers import MedicationStatementSerializer


class MedicationStatementListView(APIView):
    """medication statement list view."""

    permission_classes = [AllowAny]

    @extend_schema(responses={200: MedicationStatementSerializer(many=True)})
    def get(self, request):
        """Get medication statement list."""
        medication_statements = MedicationStatement.objects.all()
        serializer = MedicationStatementSerializer(medication_statements, many=True)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=MedicationStatementSerializer,
        responses={201: MedicationStatementSerializer},
    )
    def post(self, request):
        """Create medication statement."""
        serializer = MedicationStatementSerializer(data=request.data)
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_201_CREATED)


class MedicationStatementDetailedView(APIView):
    """medication statement detailed view."""

    def get_object(self, pk):
        """Get medication statement object."""
        try:
            return MedicationStatement.objects.get(pk=pk)

        except MedicationStatement.DoesNotExist as err:
            raise Http404 from err

    @extend_schema(responses={200, MedicationStatementSerializer})
    def get(self, request, pk):
        """Get a medication statement."""
        medication_statement = self.get_object(pk)
        serializer = MedicationStatementSerializer(medication_statement)
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(
        request=MedicationStatementSerializer,
        responses={200: MedicationStatementSerializer},
    )
    def patch(self, request, pk):
        """Update medication statement."""
        medication_statement = self.get_object(pk)
        serializer = MedicationStatementSerializer(
            medication_statement, data=request.data
        )
        serializer.is_valid(raise_exception=True)
        serializer.save()
        return Response(serializer.data, status=status.HTTP_200_OK)

    @extend_schema(responses={204: None})
    def delete(self, request, pk):
        """Delete medication statement."""
        medication_statement = self.get_object(pk)
        medication_statement.delete()
        return Response(status=status.HTTP_204_NO_CONTENT)



"""Choices for organizations."""

from django.db import models
from django.utils.translation import gettext_lazy as _


class OrganizationStatus(models.TextChoices):
    """Eligibility code choices."""

    PENDING = "pending", _("Pending")
    ACTIVE = "active", _("Active")
    INACTIVE = "inactive", _("Inactive")


class OrganizationType(models.TextChoices):
    """Eligibility code choices."""

    HEALTHCARE_PROVIDER = "healthcare_provider", _("Healthcare Provider")
    HOSPITAL_DEPARTMENT = "hospital_department", _("Hospital Department")
    ORGANIZATIONAL_TEAM = "organizational_team", _("Organizational Team")
    GOVERNMENT = "government", _("Government")
    INSURANCE_COMPANY = "insurance_company", _("Insurance Company")
    PAYER = "payer", _("Payer")
    EDUCATIONAL_INSTITUTE = "educational_institute", _("Educational Institute")
    RELIGIOUS_INSTITUTION = "religious_institution", _("Religious Institution")
    CLINICAL_RESEARCH_SPOONSOR = (
        "clinical_research_sponsor",
        _("Clinical Research Sponsor"),
    )
    COMMUNITY_GROUP = "community_group", _("Community Group")
    NON_HEALTHCARE_PROVIDER = "non_healthcare_provider", _("Non Healthcare Provider")
    OTHER = "other", _("Other")
